@@ -35,7 +35,7 @@ app.require_here()
 
 --- @usage
 local usage = [[
-ldoc, a documentation generator for Lua, vs 1.3.12
+ldoc, a documentation generator for Lua, vs 1.4.0
   -d,--dir (default doc) output directory
   -o,--output  (default 'index') output name
   -v,--verbose          verbose
@@ -51,6 +51,7 @@ ldoc, a documentation generator for Lua, vs 1.3.12
   -x,--ext (default html) output file extension
   -c,--config (default config.ld) configuration name
   -i,--ignore ignore any 'no doc comment or no module' warnings
+  -X,--not_luadoc break LuaDoc compatibility. Descriptions may continue after tags.
   -D,--define (default none) set a flag to be used in config.ld
   -C,--colon use colon style
   -B,--boilerplate ignore first comment in source files
@@ -104,6 +105,7 @@ end
 
 ProjectMap:add_kind('module','Modules')
 ProjectMap:add_kind('script','Scripts')
+ProjectMap:add_kind('classmod','Classes')
 ProjectMap:add_kind('topic','Topics')
 ProjectMap:add_kind('example','Examples')
 
@@ -117,7 +119,8 @@ local file_types = {
    ['.cpp'] = cc,
    ['.cxx'] = cc,
    ['.C'] = cc,
-   ['.mm'] = cc
+   ['.mm'] = cc,
+   ['.moon'] = lang.moon,
 }
 
 ------- ldoc external API ------------
@@ -175,20 +178,21 @@ function ldoc.new_type (tag, header, project_level,subfield)
 end
 
 function ldoc.manual_url (url)
-    global.set_manual_url(url)
+   global.set_manual_url(url)
 end
 
 function ldoc.custom_see_handler(pat, handler)
-    doc.add_custom_see_handler(pat, handler)
+   doc.add_custom_see_handler(pat, handler)
 end
 
 local ldoc_contents = {
    'alias','add_language_extension','new_type','add_section', 'tparam_alias',
    'file','project','title','package','format','output','dir','ext', 'topics',
    'one','style','template','description','examples', 'pretty', 'charset', 'plain',
-   'readme','all','manual_url', 'ignore', 'colon',
-   'boilerplate','merge', 'wrap', 'not_luadoc',
+   'readme','all','manual_url', 'ignore', 'colon', 'sort', 'module_file',
+   'boilerplate','merge', 'wrap', 'not_luadoc', 'template_escape',
    'no_return_or_parms','no_summary','full_description','backtick_references', 'custom_see_handler',
+   'no_space_before_args',
 }
 ldoc_contents = tablex.makeset(ldoc_contents)
 
@@ -280,7 +284,6 @@ if args.file == '.' then
    elseif type(args.file) == 'table' then
       for i,f in ipairs(args.file) do
          args.file[i] = abspath(f)
-         print(args.file[i])
       end
    else
       args.file = abspath(args.file)
@@ -345,7 +348,7 @@ local function process_file (f, flist)
    local ext = path.extension(f)
    local ftype = file_types[ext]
    if ftype then
-      if args.verbose then print(path.basename(f)) end
+      if args.verbose then print(f) end
       local F,err = parse.file(f,ftype,args)
       if err then
          if F then
@@ -364,9 +367,26 @@ setup_package_base()
 override 'colon'
 override 'merge'
 override 'not_luadoc'
+override 'module_file'
+
+-- ldoc.module_file establishes a partial ordering where the
+-- master module files are processed first.
+local function reorder_module_file ()
+   if args.module_file then
+      local mf = {}
+      for mname, f in pairs(args.module_file) do
+         local fullpath = abspath(f)
+         mf[fullpath] = true
+      end
+      return function(x,y)
+         return mf[x] and not mf[y]
+      end
+   end
+end
 
 if type(args.file) == 'table' then
    -- this can only be set from config file so we can assume it's already read
+   args.file.sortfn = reorder_module_file()
    process_file_list(args.file,'*.*',process_file, file_list)
    if #file_list == 0 then quit "no source files specified" end
 elseif path.isdir(args.file) then
@@ -383,9 +403,13 @@ elseif path.isdir(args.file) then
          end
       end
    end
+   -- process files, optionally in order that respects master module files
+   local sortfn = reorder_module_file()
+   if sortfn then files:sort(sortfn) end
    for f in files:iter() do
       process_file(f, file_list)
    end
+
    if #file_list == 0 then
       quit(quote(args.file).." contained no source files")
    end
@@ -553,9 +577,20 @@ end
 
 ldoc.css, ldoc.templ = 'ldoc.css','ldoc.ltp'
 
+if args.ext == 'md' then
+   ldoc.templ = 'ldoc.mdtp'
+   ldoc.template_escape = '>'
+   ldoc.style = false
+   args.ext = '.md'
+end
+
 local function style_dir (sname)
    local style = ldoc[sname]
    local dir
+   if style==false and sname == 'style' then
+      args.style = false
+      ldoc.css = false
+   end
    if style then
       if style == true then
          dir = config_dir
@@ -604,11 +639,11 @@ if args.style == '!' or args.template == '!' then
       utils.writefile(path.join(tmpdir,name),require('ldoc.html.'..name:gsub('%.','_')))
    end
    if args.style == '!' then
-      tmpwrite(ldoc.templ)
+      tmpwrite(ldoc.css)
       args.style = tmpdir
    end
    if args.template == '!' then
-      tmpwrite(ldoc.css)
+      tmpwrite(ldoc.templ)
       args.template = tmpdir
    end
 end
